/*
 * e-mail-config-ews-ooo-page.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the program; if not, see <http://www.gnu.org/licenses/>
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "e-mail-config-ews-ooo-page.h"

#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>

#include <e-util/e-util.h>
#include <e-util/e-dialog-utils.h>
#include <misc/e-dateedit.h>

#include "server/e-ews-connection.h"
#include "server/e-ews-oof-settings.h"

#include "server/e-ews-gnome-3-8-compat.h"

#include "e-ews-config-utils.h"

#define E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE(obj) \
	(G_TYPE_INSTANCE_GET_PRIVATE \
	((obj), E_TYPE_MAIL_CONFIG_EWS_OOO_PAGE, EMailConfigEwsOooPagePrivate))

struct _EMailConfigEwsOooPagePrivate {
	EAccount *account;

	/* The try_password() method deposits results here.
	 * This avoids calling GTK+ functions from multiple threads. */
	EEwsOofSettings *oof_settings;
	GMutex *oof_settings_lock;

	/* to not save unchanged state */
	gboolean changed;

	GtkWidget *enabled_radio_button;	/* not referenced */
	GtkWidget *disabled_radio_button;	/* not referenced */
	GtkWidget *scheduled_radio_button;	/* not referenced */
	GtkWidget *start_time;			/* not referenced */
	GtkWidget *end_time;			/* not referenced */
	GtkWidget *external_audience;		/* not referenced */
	GtkTextBuffer *internal_reply;		/* not referenced */
	GtkTextBuffer *external_reply;		/* not referenced */
};

enum {
	PROP_0,
	PROP_ACCOUNT
};

gchar *org_gnome_ews_get_password (EAccount *account, gboolean reask, const GError *error);

G_DEFINE_TYPE (
	EMailConfigEwsOooPage,
	e_mail_config_ews_ooo_page,
	GTK_TYPE_VBOX)

static const gchar *
mail_config_ews_ooo_page_get_mailbox (EMailConfigEwsOooPage *page)
{
	return e_account_get_string (page->priv->account, E_ACCOUNT_ID_ADDRESS);
}

static void
mail_config_ews_ooo_page_display_settings (EMailConfigEwsOooPage *page,
                                           EEwsOofSettings *oof_settings)
{
	GtkWidget *button;
	GDateTime *date_time;

	switch (e_ews_oof_settings_get_state (oof_settings)) {
		default:
			/* fall through */
		case E_EWS_OOF_STATE_DISABLED:
			button = page->priv->disabled_radio_button;
			break;
		case E_EWS_OOF_STATE_ENABLED:
			button = page->priv->enabled_radio_button;
			break;
		case E_EWS_OOF_STATE_SCHEDULED:
			button = page->priv->scheduled_radio_button;
			break;
	}

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);

	gtk_combo_box_set_active (
		GTK_COMBO_BOX (page->priv->external_audience),
		e_ews_oof_settings_get_external_audience (oof_settings));

	date_time = e_ews_oof_settings_ref_start_time (oof_settings);
	e_date_edit_set_time (
		E_DATE_EDIT (page->priv->start_time),
		(time_t) g_date_time_to_unix (date_time));
	g_date_time_unref (date_time);

	date_time = e_ews_oof_settings_ref_end_time (oof_settings);
	e_date_edit_set_time (
		E_DATE_EDIT (page->priv->end_time),
		(time_t) g_date_time_to_unix (date_time));
	g_date_time_unref (date_time);

	gtk_text_buffer_set_text (
		page->priv->internal_reply,
		e_ews_oof_settings_get_internal_reply (oof_settings), -1);

	gtk_text_buffer_set_text (
		page->priv->external_reply,
		e_ews_oof_settings_get_external_reply (oof_settings), -1);

	gtk_widget_set_sensitive (page->priv->enabled_radio_button, TRUE);
	gtk_widget_set_sensitive (page->priv->disabled_radio_button, TRUE);
	gtk_widget_set_sensitive (page->priv->scheduled_radio_button, TRUE);
}

static void
mail_config_ews_ooo_page_refresh_cb (GObject *source_object,
                                     GAsyncResult *result,
                                     gpointer user_data)
{
	EMailConfigEwsOooPage *page = user_data;
	EEwsOofSettings *oof_settings;
	GError *local_error = NULL;

	g_return_if_fail (E_IS_MAIL_CONFIG_EWS_OOO_PAGE (page));

	oof_settings = e_ews_oof_settings_new_finish (result, &local_error);

	if (oof_settings != NULL) {
		g_warn_if_fail (local_error == NULL);

		/* The page takes ownership of the settings. */
		g_mutex_lock (page->priv->oof_settings_lock);
		if (page->priv->oof_settings != NULL)
			g_object_unref (page->priv->oof_settings);
		page->priv->oof_settings = oof_settings;
		page->priv->changed = FALSE;
		mail_config_ews_ooo_page_display_settings (
			page, page->priv->oof_settings);
		g_mutex_unlock (page->priv->oof_settings_lock);

	} else if (g_error_matches (local_error, SOUP_HTTP_ERROR, SOUP_STATUS_UNAUTHORIZED)) {
		e_mail_config_ews_ooo_page_refresh (page, local_error);
	} else if (local_error) {
		GtkWindow *parent = e_ews_config_utils_get_widget_toplevel_window (GTK_WIDGET (page));

		e_notice (parent, GTK_MESSAGE_ERROR, _("Retrieving Out of Office settings failed: %s"), local_error->message);
	}

	g_clear_error (&local_error);
	g_object_unref (page);
}

static void
mail_config_ews_ooo_page_set_account (EMailConfigEwsOooPage *page,
				      EAccount *account)
{
	g_return_if_fail (E_IS_ACCOUNT (account));
	g_return_if_fail (page->priv->account == NULL);

	page->priv->account = g_object_ref (account);
}

static void
mail_config_ews_ooo_page_set_property (GObject *object,
                                       guint property_id,
                                       const GValue *value,
                                       GParamSpec *pspec)
{
	switch (property_id) {
		case PROP_ACCOUNT:
			mail_config_ews_ooo_page_set_account (
				E_MAIL_CONFIG_EWS_OOO_PAGE (object),
				g_value_get_object (value));
			return;
	}

	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
}

static void
mail_config_ews_ooo_page_get_property (GObject *object,
                                       guint property_id,
                                       GValue *value,
                                       GParamSpec *pspec)
{
	switch (property_id) {
		case PROP_ACCOUNT:
			g_value_set_object (
				value,
				e_mail_config_ews_ooo_page_get_account (
				E_MAIL_CONFIG_EWS_OOO_PAGE (object)));
			return;
	}

	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
}

static void
mail_config_ews_ooo_page_dispose (GObject *object)
{
	EMailConfigEwsOooPagePrivate *priv;

	priv = E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE (object);

	if (priv->account != NULL) {
		g_object_unref (priv->account);
		priv->account = NULL;
	}

	if (priv->oof_settings != NULL) {
		g_object_unref (priv->oof_settings);
		priv->oof_settings = NULL;
	}

	/* Chain up to parent's dispose() method. */
	if (G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->dispose)
		G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->dispose (object);
}

static void
mail_config_ews_ooo_page_finalize (GObject *object)
{
	EMailConfigEwsOooPagePrivate *priv;

	priv = E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE (object);

	g_mutex_free (priv->oof_settings_lock);

	/* Chain up to parent's finalize() method. */
	if (G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->finalize)
		G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->finalize (object);
}

static void
mail_config_ews_ooo_page_constructed (GObject *object)
{
	EMailConfigEwsOooPage *page;
	GtkLabel *label;
	GtkTable *table;
	GtkWidget *widget;
	GtkWidget *container;
	GtkSizeGroup *size_group;
	GtkTextBuffer *text_buffer;
	GSList *group = NULL;
	const gchar *text;
	gchar *markup;
	gint row = 0;

	page = E_MAIL_CONFIG_EWS_OOO_PAGE (object);

	/* Chain up to parent's constructed() method. */
	if (G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->constructed)
		G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->constructed (object);

	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

	gtk_box_set_spacing (GTK_BOX (page), 12);
	gtk_container_set_border_width (GTK_CONTAINER (page), 12);

	widget = gtk_table_new (5, 2, FALSE);
	gtk_box_pack_start (GTK_BOX (page), widget, FALSE, TRUE, 0);
	gtk_widget_show (widget);

	table = GTK_TABLE (widget);

	gtk_table_set_row_spacings (table, 6);
	gtk_table_set_col_spacings (table, 6);

	text = _("Out of Office");
	markup = g_markup_printf_escaped ("<b>%s</b>", text);
	widget = gtk_label_new (markup);
	gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
	gtk_table_attach (table, widget, 0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_widget_show (widget);
	g_free (markup);

	row++;

	text = _("The messages specified below will be automatically sent to "
		 "each internal and external person who sends a mail to you.");
	widget = gtk_label_new (text);
	gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
	gtk_table_attach (table, widget, 0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 12, 0);
	gtk_widget_show (widget);

	row++;

	text = _("Do _not send Out of Office replies");
	widget = gtk_radio_button_new_with_mnemonic (group, text);
	gtk_table_attach (table, widget, 0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 12, 0);
	page->priv->disabled_radio_button = widget;  /* do not reference */
	gtk_widget_show (widget);

	row++;

	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (widget));

	text = _("_Send Out of Office replies");
	widget = gtk_radio_button_new_with_mnemonic (group, text);
	gtk_table_attach (table, widget, 0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 12, 0);
	page->priv->enabled_radio_button = widget;  /* do not reference */
	gtk_widget_show (widget);

	row++;

	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (widget));

	text = _("Send Out of Office replies only _during this time period:");
	widget = gtk_radio_button_new_with_mnemonic (group, text);
	gtk_table_attach (table, widget, 0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 12, 0);
	page->priv->scheduled_radio_button = widget;  /* do not reference */
	gtk_widget_show (widget);

	widget = gtk_table_new (5, 2, FALSE);
	gtk_box_pack_start (GTK_BOX (page), widget, TRUE, TRUE, 0);
	gtk_widget_show (widget);

	table = GTK_TABLE (widget);
	row = 0;

	gtk_table_set_row_spacings (table, 6);
	gtk_table_set_col_spacings (table, 6);

	g_object_bind_property (
		page->priv->disabled_radio_button, "active",
		widget, "sensitive",
		G_BINDING_SYNC_CREATE |
		G_BINDING_INVERT_BOOLEAN);

	widget = gtk_label_new_with_mnemonic (_("_From:"));
	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
	gtk_table_attach (table, widget, 0, 1, row, row + 1, GTK_FILL, 0, 0, 0);
	gtk_size_group_add_widget (size_group, widget);
	gtk_widget_show (widget);

	g_object_bind_property (
		page->priv->scheduled_radio_button, "active",
		widget, "sensitive",
		G_BINDING_SYNC_CREATE);

	label = GTK_LABEL (widget);

	widget = e_date_edit_new ();
	gtk_label_set_mnemonic_widget (label, widget);
	gtk_table_attach (table, widget, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	page->priv->start_time = widget;  /* do not reference */
	gtk_widget_show (widget);

	row++;

	g_object_bind_property (
		page->priv->scheduled_radio_button, "active",
		widget, "sensitive",
		G_BINDING_SYNC_CREATE);

	widget = gtk_label_new_with_mnemonic (_("_To:"));
	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
	gtk_table_attach (table, widget, 0, 1, row, row + 1, GTK_FILL, 0, 0, 0);
	gtk_size_group_add_widget (size_group, widget);
	gtk_widget_show (widget);

	g_object_bind_property (
		page->priv->scheduled_radio_button, "active",
		widget, "sensitive",
		G_BINDING_SYNC_CREATE);

	label = GTK_LABEL (widget);

	widget = e_date_edit_new ();
	gtk_label_set_mnemonic_widget (label, widget);
	gtk_table_attach (table, widget, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	page->priv->end_time = widget;  /* do not reference */
	gtk_widget_show (widget);

	g_object_bind_property (
		page->priv->scheduled_radio_button, "active",
		widget, "sensitive",
		G_BINDING_SYNC_CREATE);

	row++;

	widget = gtk_label_new_with_mnemonic (_("I_nternal:"));
	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.0);
	gtk_table_attach (table, widget, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_size_group_add_widget (size_group, widget);
	gtk_widget_show (widget);

	text = _("Message to be sent within the organization");
	gtk_widget_set_tooltip_text (widget, text);

	label = GTK_LABEL (widget);

	widget = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (widget),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (
		GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
	gtk_table_attach (table, widget, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	gtk_widget_show (widget);

	container = widget;

	widget = gtk_text_view_new ();
	gtk_label_set_mnemonic_widget (label, widget);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (widget), GTK_WRAP_WORD);
	gtk_container_add (GTK_CONTAINER (container), widget);
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
	page->priv->internal_reply = text_buffer;  /* do not reference */
	gtk_widget_show (widget);

	row++;

	widget = gtk_label_new_with_mnemonic (_("E_xternal:"));
	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
	gtk_table_attach (table, widget, 0, 1, row, row + 1, GTK_FILL, 0, 0, 0);
	gtk_size_group_add_widget (size_group, widget);
	gtk_widget_show (widget);

	text = _("Message to be sent outside the organization");
	gtk_widget_set_tooltip_text (widget, text);

	label = GTK_LABEL (widget);

	/* The order of the combo box items needs to stay
	 * syncrhonized with the EEwsExternalAudience enum. */
	widget = gtk_combo_box_new_text ();
	gtk_combo_box_append_text (
		GTK_COMBO_BOX (widget),
		_("Do not reply to senders outside the organization"));
	gtk_combo_box_append_text (
		GTK_COMBO_BOX (widget),
		_("Reply only to known senders outside the organization"));
	gtk_combo_box_append_text (
		GTK_COMBO_BOX (widget),
		_("Reply to any sender outside the organization"));
	gtk_table_attach (table, widget, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	page->priv->external_audience = widget;  /* do not reference */
	gtk_widget_show (widget);

	row++;

	widget = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (widget),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (
		GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
	gtk_table_attach (table, widget, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	gtk_widget_show (widget);

	container = widget;

	widget = gtk_text_view_new ();
	gtk_label_set_mnemonic_widget (label, widget);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (widget), GTK_WRAP_WORD);
	gtk_container_add (GTK_CONTAINER (container), widget);
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
	page->priv->external_reply = text_buffer;  /* do not reference */
	gtk_widget_show (widget);

	/* XXX Bit of a hack.  Since the enum value for "none" is zero,
	 *     sensitize the text view if the combo box has a non-zero
	 *     "active" value (in other words, anything but "none"). */
	g_object_bind_property (
		page->priv->external_audience, "active",
		widget, "sensitive",
		G_BINDING_SYNC_CREATE);

	g_object_unref (size_group);

	gtk_widget_set_sensitive (page->priv->enabled_radio_button, FALSE);
	gtk_widget_set_sensitive (page->priv->disabled_radio_button, FALSE);
	gtk_widget_set_sensitive (page->priv->scheduled_radio_button, FALSE);

	if (e_ews_config_utils_is_online ())
		e_mail_config_ews_ooo_page_refresh (page, NULL);
}

/* Helper for mail_config_ews_ooo_page_submit() */
static void
mail_config_ews_ooo_page_submit_cb (GObject *source_object,
                                    GAsyncResult *result,
                                    gpointer user_data)
{
	GSimpleAsyncResult *simple;
	GError *error = NULL;

	simple = G_SIMPLE_ASYNC_RESULT (user_data);

	e_ews_oof_settings_submit_finish (
		E_EWS_OOF_SETTINGS (source_object), result, &error);

	if (error != NULL)
		g_simple_async_result_take_error (simple, error);

	g_simple_async_result_complete (simple);

	g_object_unref (simple);
}

static void
ews_oof_settings_changed (gboolean *pchanged)
{
	g_return_if_fail (pchanged != NULL);

	*pchanged = TRUE;
}

static void
mail_config_ews_ooo_page_submit (EMailConfigEwsOooPage *page,
                                 GCancellable *cancellable,
                                 GAsyncReadyCallback callback,
                                 gpointer user_data)
{
	EMailConfigEwsOooPagePrivate *priv;
	GSimpleAsyncResult *simple;
	GtkToggleButton *toggle_button;
	GtkTextBuffer *text_buffer;
	GtkTextIter start, end;
	GDateTime *date_time;
	gchar *text;
	time_t tm;
	gulong signal_id;

	priv = E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE (page);

	g_mutex_lock (priv->oof_settings_lock);

	/* It may be that the Out of Office settings are still
	 * loading or have failed to load, in which case there
	 * are obviously no changes to submit. */
	if (priv->oof_settings == NULL) {
		g_mutex_unlock (priv->oof_settings_lock);

		simple = g_simple_async_result_new (
			G_OBJECT (page), callback, user_data,
			mail_config_ews_ooo_page_submit);
		g_simple_async_result_complete_in_idle (simple);
		g_object_unref (simple);

		return;
	}

	signal_id = g_signal_connect_swapped (
		priv->oof_settings, "notify",
		G_CALLBACK (ews_oof_settings_changed), &priv->changed);

	toggle_button = GTK_TOGGLE_BUTTON (priv->enabled_radio_button);
	if (gtk_toggle_button_get_active (toggle_button))
		e_ews_oof_settings_set_state (
			priv->oof_settings,
			E_EWS_OOF_STATE_ENABLED);

	toggle_button = GTK_TOGGLE_BUTTON (priv->disabled_radio_button);
	if (gtk_toggle_button_get_active (toggle_button))
		e_ews_oof_settings_set_state (
			priv->oof_settings,
			E_EWS_OOF_STATE_DISABLED);

	toggle_button = GTK_TOGGLE_BUTTON (priv->scheduled_radio_button);
	if (gtk_toggle_button_get_active (toggle_button))
		e_ews_oof_settings_set_state (
			priv->oof_settings,
			E_EWS_OOF_STATE_SCHEDULED);

	tm = e_date_edit_get_time (E_DATE_EDIT (priv->start_time));
	date_time = g_date_time_new_from_unix_utc ((gint64) tm);
	e_ews_oof_settings_set_start_time (priv->oof_settings, date_time);
	g_date_time_unref (date_time);

	tm = e_date_edit_get_time (E_DATE_EDIT (priv->end_time));
	date_time = g_date_time_new_from_unix_utc ((gint64) tm);
	e_ews_oof_settings_set_end_time (priv->oof_settings, date_time);
	g_date_time_unref (date_time);

	text_buffer = priv->internal_reply;
	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
	text = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
	e_ews_oof_settings_set_internal_reply (priv->oof_settings, text);
	g_free (text);

	text_buffer = priv->external_reply;
	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
	text = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
	e_ews_oof_settings_set_external_reply (priv->oof_settings, text);
	g_free (text);

	simple = g_simple_async_result_new (
		G_OBJECT (page), callback, user_data,
		mail_config_ews_ooo_page_submit);

	g_signal_handler_disconnect (priv->oof_settings, signal_id);

	if (priv->changed) {
		g_simple_async_result_set_check_cancellable (simple, cancellable);

		e_ews_oof_settings_submit (
			priv->oof_settings, cancellable,
			mail_config_ews_ooo_page_submit_cb,
			g_object_ref (simple));
	} else {
		g_simple_async_result_complete_in_idle (simple);
	}

	g_object_unref (simple);

	g_mutex_unlock (priv->oof_settings_lock);
}

static gboolean
mail_config_ews_ooo_page_submit_finish (EMailConfigEwsOooPage *page,
                                        GAsyncResult *result,
                                        GError **error)
{
	GSimpleAsyncResult *simple;

	g_return_val_if_fail (
		g_simple_async_result_is_valid (
		result, G_OBJECT (page),
		mail_config_ews_ooo_page_submit), FALSE);

	simple = G_SIMPLE_ASYNC_RESULT (result);

	/* Assume success unless a GError is set. */
	return !g_simple_async_result_propagate_error (simple, error);
}

static void
e_mail_config_ews_ooo_page_class_init (EMailConfigEwsOooPageClass *class)
{
	GObjectClass *object_class;

	g_type_class_add_private (
		class, sizeof (EMailConfigEwsOooPagePrivate));

	object_class = G_OBJECT_CLASS (class);
	object_class->set_property = mail_config_ews_ooo_page_set_property;
	object_class->get_property = mail_config_ews_ooo_page_get_property;
	object_class->dispose = mail_config_ews_ooo_page_dispose;
	object_class->finalize = mail_config_ews_ooo_page_finalize;
	object_class->constructed = mail_config_ews_ooo_page_constructed;

	g_object_class_install_property (
		object_class,
		PROP_ACCOUNT,
		g_param_spec_object (
			"account",
			"Account",
			"Mail account being edited",
			E_TYPE_ACCOUNT,
			G_PARAM_READWRITE |
			G_PARAM_CONSTRUCT_ONLY));
}

static void
e_mail_config_ews_ooo_page_init (EMailConfigEwsOooPage *page)
{
	page->priv = E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE (page);

	page->priv->oof_settings_lock = g_mutex_new ();
}

GtkWidget *
e_mail_config_ews_ooo_page_new (EAccount *account)
{
	g_return_val_if_fail (E_IS_ACCOUNT (account), NULL);

	return g_object_new (
		E_TYPE_MAIL_CONFIG_EWS_OOO_PAGE,
		"account", account,
		NULL);
}

void
e_mail_config_ews_ooo_page_refresh (EMailConfigEwsOooPage *page,
				    const GError *with_error)
{
	EEwsConnection *connection;
	const gchar *timeout_str;
	gchar *password;
	CamelURL *url;

	g_return_if_fail (E_IS_MAIL_CONFIG_EWS_OOO_PAGE (page));
	g_return_if_fail (page->priv->account != NULL);
	
	password = org_gnome_ews_get_password (page->priv->account, with_error != NULL, with_error);
	if (!password)
		return;

	url = camel_url_new (e_account_get_string (page->priv->account, E_ACCOUNT_SOURCE_URL), NULL);
	timeout_str = camel_url_get_param (url, "timeout");
	connection = e_ews_connection_new (camel_url_get_param (url, "hosturl"),
		url->user,
		password,
		mail_config_ews_ooo_page_get_mailbox (page),
		camel_url_get_param (url, "impersonate_user"),
		timeout_str ? atoi (timeout_str) : -1,
		TRUE,
		NULL, NULL, NULL);

	g_free (password);
	password = NULL;

	if (connection) {
		e_ews_oof_settings_new (connection, G_PRIORITY_DEFAULT, NULL, mail_config_ews_ooo_page_refresh_cb, g_object_ref (page));
		g_object_unref (connection);
	}

	camel_url_free (url);
}

EAccount *
e_mail_config_ews_ooo_page_get_account (EMailConfigEwsOooPage *page)
{
	g_return_val_if_fail (E_IS_MAIL_CONFIG_EWS_OOO_PAGE (page), NULL);

	return page->priv->account;
}

void
e_mail_config_ews_ooo_page_commit (EMailConfigEwsOooPage *page,
				   GError **error)
{
	EAsyncClosure *closure;
	GAsyncResult *result;

	g_return_if_fail (E_IS_MAIL_CONFIG_EWS_OOO_PAGE (page));

	closure = e_async_closure_new ();

	mail_config_ews_ooo_page_submit (page, NULL, e_async_closure_callback, closure);

	result = e_async_closure_wait (closure);

	mail_config_ews_ooo_page_submit_finish (page, result, error);

	e_async_closure_free (closure);
}
